const config = require("config");
const needle = require("needle");
const authentication = require("../../authentication")

const capeCryptoAPISecret = config.get("cape-crypto.apiSecret");
const capeCryptoAPIKey = config.get("cape-crypto.apiKey");

module.exports.redeemVoucher = (paymentMethodId, amountInZAR, voucherPIN) => {
    const redeemVoucherURL = `https://trade.capecrypto.com/api/v2/payments/vouchers/new`;

    const requestAuthentication = authentication.generateSignature(
        capeCryptoAPISecret,
        capeCryptoAPIKey
    );
    
    const data = {
        payment_method_id: paymentMethodId,
        amount: amountInZAR,
        voucher_pin: voucherPIN
    };

    return needle(
        "POST",
        redeemVoucherURL,
        data,
        {
            json: true,
            headers: {
                "X-Auth-Nonce": requestAuthentication.nonce,
                "X-Auth-Apikey": capeCryptoAPIKey,
                "X-Auth-Signature": requestAuthentication.signature
            }
        }
    ).then(response => {
        if ([200,201].includes(response.statusCode)) {
            const redeemedAmount = amountInZAR - (amountInZAR*5.6/100)
            return Object.assign(
                {
                    redeemedAmount: redeemedAmount
                },
                response.body
            ) 
        } else {
            if (process.env.NODE_ENV != "test") {
                console.error("Failed to get tradingFees: ", response.statusMessage)
                console.log("Status code: ", response.statusCode)
                console.log("Created Body: ", response.body)
                console.log("Headers: ", response.headers)
            }

            return null
        }
    }).catch(error => {
        console.error("Error: ", error);
        return null
    })
}

const testRedemption = async () => {
    const result = await this.redeemVoucher(
        "b2191d0d-6747-486c-bc1f-6f605b93cccc",
        10,
        "1208786638013695"
    )

    console.log("Redeemed: ", result);
}

// testRedemption()