const config = require("config");
const needle = require("needle");
const authentication = require("../../authentication")

const capeCryptoAPISecret = config.get("cape-crypto.apiSecret");
const capeCryptoAPIKey = config.get("cape-crypto.apiKey");

module.exports.transactions = () => {
    const transactionURL = `https://trade.capecrypto.com/api/v2/trade/account/transactions`;

    const requestAuthentication = authentication.generateSignature(
        capeCryptoAPISecret,
        capeCryptoAPIKey
    )
    return needle(
        "GET",
        transactionURL,
        {
            json: true,
            headers: {
                "X-Auth-Nonce": requestAuthentication.nonce,
                "X-Auth-Apikey": capeCryptoAPIKey,
                "X-Auth-Signature": requestAuthentication.signature
            }
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to get tradingFees: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}

module.exports.lightning = require("./lightning");

const testTransactions = async () => {
    const result = await this.transactions();

    console.log("Result: ", result);
}
