const config = require("config");
const needle = require("needle");
const authentication = require("../../../authentication")

const capeCryptoAPISecret = config.get("cape-crypto.apiSecret");
const capeCryptoAPIKey = config.get("cape-crypto.apiKey");

module.exports.createInvoice = (amountInSatoshis, description) => {
    const createInvoiceURL = `https://trade.capecrypto.com/api/v2/trade/account/lightning/invoices`;

    const requestAuthentication = authentication.generateSignature(
        capeCryptoAPISecret,
        capeCryptoAPIKey
    )
    const data = {
        amount: amountInSatoshis,
        description
    };

    return needle(
        "POST",
        createInvoiceURL,
        data,
        {
            json: true,
            headers: {
                "X-Auth-Nonce": requestAuthentication.nonce,
                "X-Auth-Apikey": capeCryptoAPIKey,
                "X-Auth-Signature": requestAuthentication.signature
            }
        }
    ).then(response => {
        if ([200,201].includes(response.statusCode)) {
            return response.body
        } else {
            console.error("Failed to get tradingFees: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}

module.exports.payInvoice = (bolt11Invoice, amountInSatoshis, description) => {
    const payInvoiceURL = `https://trade.capecrypto.com/api/v2/trade/account/lightning/pay`;

    const requestAuthentication = authentication.generateSignature(
        capeCryptoAPISecret,
        capeCryptoAPIKey
    );

    const data = {
        bolt: bolt11Invoice,
        amount: amountInSatoshis,
        member_description: description
    };

    return needle(
        "POST",
        payInvoiceURL,
        data,
        {
            json: true,
            headers: {
                "X-Auth-Nonce": requestAuthentication.nonce,
                "X-Auth-Apikey": capeCryptoAPIKey,
                "X-Auth-Signature": requestAuthentication.signature
            }
        }
    ).then(response => {
        if ([200,201].includes(response.statusCode)) {
            return response.body
        } else {
            console.error("Failed to get tradingFees: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}
