const config = require("config");
const needle = require("needle");
const authentication = require("../../authentication")

const capeCryptoAPISecret = config.get("cape-crypto.apiSecret");
const capeCryptoAPIKey = config.get("cape-crypto.apiKey");

module.exports.trades = () => {
    const createInvoiceURL = `https://trade.capecrypto.com/api/v2/trade/market/trades`;

    const requestAuthentication = authentication.generateSignature(
        capeCryptoAPISecret,
        capeCryptoAPIKey
    )
    return needle(
        "GET",
        createInvoiceURL,
        {
            json: true,
            headers: {
                "X-Auth-Nonce": requestAuthentication.nonce,
                "X-Auth-Apikey": capeCryptoAPIKey,
                "X-Auth-Signature": requestAuthentication.signature
            }
        }
    ).then(response => {
        if ([200,201].includes(response.statusCode)) {
            return response.body
        } else {
            console.error("Failed to get tradingFees: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}

module.exports.buyBitcoin = (amountInBTC) => {
    const payInvoiceURL = `https://trade.capecrypto.com/api/v2/trade/market/orders`;

    const requestAuthentication = authentication.generateSignature(
        capeCryptoAPISecret,
        capeCryptoAPIKey
    );

    const data = {
        market: "btczar",
        side: "buy",
        volume: amountInBTC,
        ord_type: "market",
        // price: 100
    };

    return needle(
        "POST",
        payInvoiceURL,
        data,
        {
            json: true,
            headers: {
                "X-Auth-Nonce": requestAuthentication.nonce,
                "X-Auth-Apikey": capeCryptoAPIKey,
                "X-Auth-Signature": requestAuthentication.signature
            }
        }
    ).then(response => {
        if ([200,201].includes(response.statusCode)) {
            return response.body
        } else {
            console.error("Failed to get tradingFees: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}

const testTrades = async () => {

    const result = await this.trades();
    console.log("Trades: ", result);
}

const testBuy = async () => {
    const result = await this.buyBitcoin(
        0.00000063
    );

    console.log("Bought: ", result);
}

// testBuy();