const config = require("config")
const needle = require('needle');

const NodeCache = require( "node-cache" ); // TODO: Reimplement cache

var ttl = 60*15

const coinmarketcapAPIBaseUrl = config.get('coinmarketcap.baseAPIURL')
const coinmarketcapAPIKey = config.get('coinmarketcap.api-key')

if (process.env.NODE_ENV != 'production') {
    ttl = 60
}

const bitcoinExchangeRateCache = new NodeCache({
    stdTTL: ttl,
    checkperiod: ttl/10,
    deleteOnExpire: false
})

/**
 * Cache expires after 60 seconds and we need to get a fresh value...
 */
bitcoinExchangeRateCache.on( "expired", function( key, value ){
    // Depending on some metric just set the value to the key again
    fetchExchangeRate(key).then(freshBitcoinExchangeRate => {
        const randomTTL = Math.ceil(ttl*Math.random())
        bitcoinExchangeRateCache.set(key, freshBitcoinExchangeRate, randomTTL);
    })
});

/**
 * Get the Bitcoin price from coinmarket cap. ExchangeRate might be cached.
 */
module.exports.getBitcoinExchangeRate = (country, currencySymbol) => {
    const currencyTicker = country ? country.currencyTicker : currencySymbol ? currencySymbol : "BTC"
    return new Promise((resolve, reject) => {
        const cachedBitcoinExchangeRate = bitcoinExchangeRateCache.get(currencyTicker);

        if (cachedBitcoinExchangeRate == undefined) {
            fetchExchangeRate(currencyTicker).then(freshBitcoinExchangeRate => {

                const randomTTL = Math.ceil(ttl*Math.random())
                bitcoinExchangeRateCache.set(currencyTicker, freshBitcoinExchangeRate, randomTTL)
                // Could also recursively call getBitcoinExchangeRate
                resolve(freshBitcoinExchangeRate);
            }).catch(error => {
                // TODO: Use db price for the currencyTicker
                console.error("ExchangeRate Error: ", error)
            })
        } else {
            resolve(cachedBitcoinExchangeRate);
        }
    })
}

const fetchExchangeRate = (currencyTicker) => {
    return new Promise((resolve, reject) => {
        if (process.env.NODE_ENV == "test") {
            // TODO: Look at what the input is
            if (currencyTicker == "ZAR") {
                const testData = {
                    id: 1,
                    name: 'Bitcoin',
                    symbol: 'BTC',
                    slug: 'bitcoin',
                    num_market_pairs: 9628,
                    date_added: '2013-04-28T00:00:00.000Z',
                    tags: [
                      'mineable',
                      'pow',
                      'sha-256',
                      'store-of-value',
                      'state-channel',
                      'coinbase-ventures-portfolio',
                      'three-arrows-capital-portfolio',
                      'polychain-capital-portfolio',
                      'binance-labs-portfolio',
                      'blockchain-capital-portfolio',
                      'boostvc-portfolio',
                      'cms-holdings-portfolio',
                      'dcg-portfolio',
                      'dragonfly-capital-portfolio',
                      'electric-capital-portfolio',
                      'fabric-ventures-portfolio',
                      'framework-ventures-portfolio',
                      'galaxy-digital-portfolio',
                      'huobi-capital-portfolio',
                      'alameda-research-portfolio',
                      'a16z-portfolio',
                      '1confirmation-portfolio',
                      'winklevoss-capital-portfolio',
                      'usv-portfolio',
                      'placeholder-ventures-portfolio',
                      'pantera-capital-portfolio',
                      'multicoin-capital-portfolio',
                      'paradigm-portfolio'
                    ],
                    max_supply: 21000000,
                    circulating_supply: 19098825,
                    total_supply: 19098825,
                    is_active: 1,
                    platform: null,
                    cmc_rank: 1,
                    is_fiat: 0,
                    self_reported_circulating_supply: null,
                    self_reported_market_cap: null,
                    tvl_ratio: null,
                    last_updated: '2022-07-21T00:58:00.000Z',
                    quote: {
                      ZAR: {
                        price: 500000,
                        volume_24h: 1823338.8985488599,
                        volume_change_24h: -9.5311,
                        percent_change_1h: 0,
                        percent_change_24h: 0,
                        percent_change_7d: 0,
                        percent_change_30d: 0,
                        percent_change_60d: 0,
                        percent_change_90d: 0,
                        market_cap: 19097986.700993966,
                        market_cap_dominance: 42.7479,
                        fully_diluted_market_cap: 20999078.253288925,
                        tvl: null,
                        last_updated: '2022-07-21T01:00:00.000Z'
                      }
                    }
                  }
                return resolve(bitcoinExchangeRate(currencyTicker, testData))
            }
            const testData = {
                id: 1,
                name: 'Bitcoin',
                symbol: 'BTC',
                slug: 'bitcoin',
                num_market_pairs: 9628,
                date_added: '2013-04-28T00:00:00.000Z',
                tags: [
                  'mineable',
                  'pow',
                  'sha-256',
                  'store-of-value',
                  'state-channel',
                  'coinbase-ventures-portfolio',
                  'three-arrows-capital-portfolio',
                  'polychain-capital-portfolio',
                  'binance-labs-portfolio',
                  'blockchain-capital-portfolio',
                  'boostvc-portfolio',
                  'cms-holdings-portfolio',
                  'dcg-portfolio',
                  'dragonfly-capital-portfolio',
                  'electric-capital-portfolio',
                  'fabric-ventures-portfolio',
                  'framework-ventures-portfolio',
                  'galaxy-digital-portfolio',
                  'huobi-capital-portfolio',
                  'alameda-research-portfolio',
                  'a16z-portfolio',
                  '1confirmation-portfolio',
                  'winklevoss-capital-portfolio',
                  'usv-portfolio',
                  'placeholder-ventures-portfolio',
                  'pantera-capital-portfolio',
                  'multicoin-capital-portfolio',
                  'paradigm-portfolio'
                ],
                max_supply: 21000000,
                circulating_supply: 19098825,
                total_supply: 19098825,
                is_active: 1,
                platform: null,
                cmc_rank: 1,
                is_fiat: 0,
                self_reported_circulating_supply: null,
                self_reported_market_cap: null,
                tvl_ratio: null,
                last_updated: '2022-07-21T00:58:00.000Z',
                quote: {
                  BTC: {
                    price: 1,
                    volume_24h: 1823338.8985488599,
                    volume_change_24h: -9.5311,
                    percent_change_1h: 0,
                    percent_change_24h: 0,
                    percent_change_7d: 0,
                    percent_change_30d: 0,
                    percent_change_60d: 0,
                    percent_change_90d: 0,
                    market_cap: 19097986.700993966,
                    market_cap_dominance: 42.7479,
                    fully_diluted_market_cap: 20999078.253288925,
                    tvl: null,
                    last_updated: '2022-07-21T01:00:00.000Z'
                  }
                }
              }
            return resolve(bitcoinExchangeRate(currencyTicker, testData))
        }
        needle(
            "GET",
            `${coinmarketcapAPIBaseUrl}/v1/cryptocurrency/quotes/latest`, 
            {
                symbol: 'BTC', 
                convert: currencyTicker
            },
            {
                // json: true,
                headers: {
                    'X-CMC_PRO_API_KEY': coinmarketcapAPIKey,
                    'Accept': 'application/json'
                }
            }
        ).then(response => {

            if (response.statusCode == 200) {
                return response.body
            } else {
                console.log("Status code: ", response.statusCode)
                console.log("Status message: ", response.statusMessage)
                console.log("Status message: ", response.body)
                return null
            }
        }).then(body => {
            if (body && body.status.error_code == 0) {
                resolve(bitcoinExchangeRate(currencyTicker, body.data["BTC"]))
            } else {
                reject("Failed to get response from coinmarketcap")
            }
        }).catch(reason => {
            console.error("Something went wrong: ", reason);
            reject(reason)
        })
    })
}

const bitcoinExchangeRate = async (currencyTicker, coinmarketcapBitcoinExchangeRate) => {
    if (coinmarketcapBitcoinExchangeRate) {
        // TODO: Make this support multiple currencies...
        var quotes;
        const coinMarketcapCurrencyTicker = Object.keys(coinmarketcapBitcoinExchangeRate.quote)[0]
        quotes = coinmarketcapBitcoinExchangeRate.quote[coinMarketcapCurrencyTicker];
        quotes.currency = currencyTicker;

        if (quotes) {
            return {
                maxSupply: coinmarketcapBitcoinExchangeRate.max_supply,
                circulatingSupply: coinmarketcapBitcoinExchangeRate.circulating_supply,
                marketCap: quotes.market_cap,
                currency: quotes.currency,
                last_updated: quotes.last_updated,
                marketExchangeRate: moneyFormat(quotes.price), 
                marketValue: quotes.price, 
                percent_change_1h: quotes.percent_change_1h,
                percent_change_24h: quotes.percent_change_24h,
                percent_change_7d: quotes.percent_change_7d
                // add last updated...
            }
        }
        return quotes;
    } else {
        return null;
    }
}

const moneyFormat = (amount) => {
    return Number(amount).toLocaleString("za", {
        minimumFractionDigits: 2,
        maximumFractionDigits: 2
    });
}

// const function async () 
// this.getBitcoinExchangeRate({
//     currencyTicker: "BTC"
// }).then(bitcoinZAR => {
//     console.log(`${bitcoinZAR.currency}: `, bitcoinZAR)
// })

