const needle = require('needle');

const NodeCache = require( "node-cache" ); // TODO: Reimplement cache

var ttl = 60*15

if (process.env.NODE_ENV != 'production') {
    ttl = 60
}

const CACHE_KEY = "bitnob"

const bitnobExchangeRateCache = new NodeCache({
    stdTTL: ttl,
    checkperiod: ttl/10,
    deleteOnExpire: false
})

/**
 * Cache expires after 60 seconds and we need to get a fresh value...
 */
bitnobExchangeRateCache.on( "expired", function( key, value ){
    // Depending on some metric just set the value to the key again
    fetchBitnobExchangeRates().then(freshBitnobExchangeRate => {
        bitnobExchangeRateCache.set(key, freshBitnobExchangeRate, ttl);
    })
});

/**
 * Get the Bitcoin price from coinmarket cap. ExchangeRate might be cached.
 */
module.exports.getBitcoinExchangeRate = (country, currencySymbol) => {
    const currencyTicker = country ? country.currencyTicker : currencySymbol ? currencySymbol : "BTC"
    return new Promise(async (resolve, reject) => {
        const cachedBitnobExchangeRate = bitnobExchangeRateCache.get(CACHE_KEY);

        if (cachedBitnobExchangeRate == undefined) {
            fetchBitnobExchangeRates().then(async (freshBitnobExchangeRate) => {
                bitnobExchangeRateCache.set(CACHE_KEY, freshBitnobExchangeRate, ttl)
                // Could also recursively call getBitcoinExchangeRate
                resolve(bitnobExchangeRate(currencyTicker, freshBitnobExchangeRate));
            }).catch(error => {
                // TODO: Use db price for the currencyTicker
                console.error("ExchangeRate Error: ", error)
            })  
        } else {
            resolve(bitnobExchangeRate(currencyTicker, cachedBitnobExchangeRate));
        }
    })
}

const fetchBitnobExchangeRates = async () => {
    if (process.env.NODE_ENV == "test") {
        // TODO: Look at what the input is
        const testData = JSON.parse(
            `{"status":true,"message":"successfully fetched all exchange rates","data":[{"code":"BTC","name":"Bitcoin","rate":1},{"code":"BCH","name":"Bitcoin Cash","rate":163.41},{"code":"USD","name":"US Dollar","rate":24364.86},{"code":"EUR","name":"Eurozone Euro","rate":23813.56},{"code":"GBP","name":"Pound Sterling","rate":20012.2},{"code":"JPY","name":"Japanese Yen","rate":3245521.62},{"code":"CAD","name":"Canadian Dollar","rate":31175.45},{"code":"AUD","name":"Australian Dollar","rate":34845.45},{"code":"CNY","name":"Chinese Yuan","rate":164321.49},{"code":"CHF","name":"Swiss Franc","rate":23199.76},{"code":"SEK","name":"Swedish Krona","rate":247181.51},{"code":"NZD","name":"New Zealand Dollar","rate":38800.72},{"code":"KRW","name":"South Korean Won","rate":31754235.72},{"code":"ETH","name":"Ether","rate":14.19},{"code":"LTC","name":"Litecoin","rate":388.84},{"code":"XRP","name":"Ripple","rate":61578.74},{"code":"AED","name":"UAE Dirham","rate":89494.57},{"code":"AFN","name":"Afghan Afghani","rate":2192837.47},{"code":"ALL","name":"Albanian Lek","rate":2786730.95},{"code":"AMD","name":"Armenian Dram","rate":9916648.29},{"code":"ANG","name":"Netherlands Antillean Guilder","rate":43976.8},{"code":"AOA","name":"Angolan Kwanza","rate":10541257.21},{"code":"ARS","name":"Argentine Peso","rate":3196810.75},{"code":"AWG","name":"Aruban Florin","rate":43856.75},{"code":"AZN","name":"Azerbaijani Manat","rate":41420.26},{"code":"BAM","name":"Bosnia-Herzegovina Convertible Mark","rate":46600.82},{"code":"BBD","name":"Barbadian Dollar","rate":48729.72},{"code":"BDT","name":"Bangladeshi Taka","rate":2310195.72},{"code":"BGN","name":"Bulgarian Lev","rate":46642.63},{"code":"BHD","name":"Bahraini Dinar","rate":9181.22},{"code":"BIF","name":"Burundian Franc","rate":49728680.79},{"code":"BMD","name":"Bermudan Dollar","rate":24364.86},{"code":"BND","name":"Brunei Dollar","rate":33643.19},{"code":"BOB","name":"Bolivian Boliviano","rate":168003.85},{"code":"BRL","name":"Brazilian Real","rate":126034.55},{"code":"BSD","name":"Bahamian Dollar","rate":24364.86},{"code":"BTN","name":"Bhutanese Ngultrum","rate":1944398.63},{"code":"BUSD","name":"Binance USD","rate":24364.86},{"code":"BWP","name":"Botswanan Pula","rate":305686.29},{"code":"BYN","name":"Belarusian Ruble","rate":61603.72},{"code":"BZD","name":"Belize Dollar","rate":49185.3},{"code":"CDF","name":"Congolese Franc","rate":48729721.5},{"code":"CLF","name":"Chilean Unit of Account (UF)","rate":796.9},{"code":"CLP","name":"Chilean Peso","rate":21988799.53},{"code":"COP","name":"Colombian Peso","rate":106666215.15},{"code":"CRC","name":"Costa Rican Colón","rate":16319329.65},{"code":"CUP","name":"Cuban Peso","rate":627395.16},{"code":"CVE","name":"Cape Verdean Escudo","rate":2542473.22},{"code":"CZK","name":"Czech Koruna","rate":586559.66},{"code":"DAI","name":"Dai","rate":24364.86},{"code":"DJF","name":"Djiboutian Franc","rate":4344070.35},{"code":"DKK","name":"Danish Krone","rate":177355.48},{"code":"DOGE","name":"Dogecoin","rate":342684.4},{"code":"DOP","name":"Dominican Peso","rate":1330321.4},{"code":"DZD","name":"Algerian Dinar","rate":3553529.35},{"code":"EGP","name":"Egyptian Pound","rate":460752.04},{"code":"ETB","name":"Ethiopian Birr","rate":1270627.49},{"code":"FJD","name":"Fijian Dollar","rate":53328.59},{"code":"FKP","name":"Falkland Islands Pound","rate":20012.2},{"code":"GEL","name":"Georgian Lari","rate":69439.85},{"code":"GHS","name":"Ghanaian Cedi","rate":241919.16},{"code":"GIP","name":"Gibraltar Pound","rate":20012.2},{"code":"GMD","name":"Gambian Dalasi","rate":1312656.87},{"code":"GNF","name":"Guinean Franc","rate":214695031.32},{"code":"GTQ","name":"Guatemalan Quetzal","rate":188804.5},{"code":"GUSD","name":"Gemini US Dollar","rate":24364.86},{"code":"GYD","name":"Guyanaese Dollar","rate":5105090.33},{"code":"HKD","name":"Hong Kong Dollar","rate":191263.43},{"code":"HNL","name":"Honduran Lempira","rate":598401},{"code":"HRK","name":"Croatian Kuna","rate":179527.6},{"code":"HTG","name":"Haitian Gourde","rate":2854952.44},{"code":"HUF","name":"Hungarian Forint","rate":9634734.69},{"code":"IDR","name":"Indonesian Rupiah","rate":360975157.96},{"code":"ILS","name":"Israeli Shekel","rate":83061.15},{"code":"INR","name":"Indian Rupee","rate":1929575.15},{"code":"IQD","name":"Iraqi Dinar","rate":35572696.7},{"code":"IRR","name":"Iranian Rial","rate":1030633609.73},{"code":"ISK","name":"Icelandic Króna","rate":3304849.71},{"code":"JEP","name":"Jersey Pound","rate":20012.2},{"code":"JMD","name":"Jamaican Dollar","rate":3746711.92},{"code":"JOD","name":"Jordanian Dinar","rate":17274.69},{"code":"KES","name":"Kenyan Shilling","rate":2898200.19},{"code":"KGS","name":"Kyrgystani Som","rate":1984950.87},{"code":"KHR","name":"Cambodian Riel","rate":99995119.47},{"code":"KMF","name":"Comorian Franc","rate":11320522.74},{"code":"KPW","name":"North Korean Won","rate":21928374.68},{"code":"KWD","name":"Kuwaiti Dinar","rate":7477.21},{"code":"KYD","name":"Cayman Islands Dollar","rate":20333.65},{"code":"KZT","name":"Kazakhstani Tenge","rate":11643359.07},{"code":"LAK","name":"Laotian Kip","rate":362427303.66},{"code":"LBP","name":"Lebanese Pound","rate":36985858.62},{"code":"LKR","name":"Sri Lankan Rupee","rate":8735413.16},{"code":"LRD","name":"Liberian Dollar","rate":3721731.82},{"code":"LSL","name":"Lesotho Loti","rate":385939.39},{"code":"LYD","name":"Libyan Dinar","rate":118535.05},{"code":"MAD","name":"Moroccan Dirham","rate":250808.51},{"code":"MDL","name":"Moldovan Leu","rate":470502.3},{"code":"MGA","name":"Malagasy Ariary","rate":101479645.02},{"code":"MKD","name":"Macedonian Denar","rate":1471876.83},{"code":"MMK","name":"Myanma Kyat","rate":45178967.51},{"code":"MNT","name":"Mongolian Tugrik","rate":76719098.84},{"code":"MOP","name":"Macanese Pataca","rate":197294.39},{"code":"MRU","name":"Mauritanian Ouguiya","rate":916606.06},{"code":"MUR","name":"Mauritian Rupee","rate":1109226.68},{"code":"MVR","name":"Maldivian Rufiyaa","rate":375218.86},{"code":"MWK","name":"Malawian Kwacha","rate":24852157.97},{"code":"MXN","name":"Mexican Peso","rate":496239.12},{"code":"MYR","name":"Malaysian Ringgit","rate":108435.81},{"code":"MZN","name":"Mozambican Metical","rate":1556305.53},{"code":"NAD","name":"Namibian Dollar","rate":385939.39},{"code":"NGN","name":"Nigerian Naira","rate":18012061.52},{"code":"NIO","name":"Nicaraguan Córdoba","rate":873480.26},{"code":"NOK","name":"Norwegian Krone","rate":235341.41},{"code":"NPR","name":"Nepalese Rupee","rate":3093052.95},{"code":"OMR","name":"Omani Rial","rate":9376.6},{"code":"PAB","name":"Panamanian Balboa","rate":24364.86},{"code":"PAX","name":"Paxos Standard USD","rate":24364.86},{"code":"PEN","name":"Peruvian Nuevo Sol","rate":95725.81},{"code":"PGK","name":"Papua New Guinean Kina","rate":85886.13},{"code":"PHP","name":"Philippine Peso","rate":1348728.95},{"code":"PKR","name":"Pakistani Rupee","rate":5825816.97},{"code":"PLN","name":"Polish Zloty","rate":112905.47},{"code":"PYG","name":"Paraguayan Guarani","rate":168259932.56},{"code":"QAR","name":"Qatari Rial","rate":88712.46},{"code":"RON","name":"Romanian Leu","rate":117438.63},{"code":"RSD","name":"Serbian Dinar","rate":2796233.24},{"code":"RUB","name":"Russian Ruble","rate":1516712.68},{"code":"RWF","name":"Rwandan Franc","rate":25071441.71},{"code":"SAR","name":"Saudi Riyal","rate":91517.32},{"code":"SBD","name":"Solomon Islands Dollar","rate":200044.77},{"code":"SCR","name":"Seychellois Rupee","rate":326913.98},{"code":"SDG","name":"Sudanese Pound","rate":11122558.93},{"code":"SGD","name":"Singapore Dollar","rate":33637.15},{"code":"SHP","name":"Saint Helena Pound","rate":20012.2},{"code":"SLL","name":"Sierra Leonean Leone","rate":334990033.97},{"code":"SOS","name":"Somali Shilling","rate":14253443.54},{"code":"SRD","name":"Surinamese Dollar","rate":590141.29},{"code":"STN","name":"São Tomé and Príncipe Dobra","rate":572574.23},{"code":"SVC","name":"Salvadoran Colón","rate":213510.32},{"code":"SYP","name":"Syrian Pound","rate":61217443.58},{"code":"SZL","name":"Swazi Lilangeni","rate":402243.34},{"code":"THB","name":"Thai Baht","rate":883559.15},{"code":"TJS","name":"Tajikistani Somoni","rate":248282.9},{"code":"TMT","name":"Turkmenistani Manat","rate":85277.01},{"code":"TND","name":"Tunisian Dinar","rate":76371.66},{"code":"TOP","name":"Tongan Paʻanga","rate":56937.63},{"code":"TRY","name":"Turkish Lira","rate":436491.61},{"code":"TTD","name":"Trinidad and Tobago Dollar","rate":165733.19},{"code":"TWD","name":"New Taiwan Dollar","rate":731238.15},{"code":"TZS","name":"Tanzanian Shilling","rate":56770125.55},{"code":"UAH","name":"Ukrainian Hryvnia","rate":900710.5},{"code":"UGX","name":"Ugandan Shilling","rate":94188828.94},{"code":"USDC","name":"Circle USD Coin","rate":24364.86},{"code":"UYU","name":"Uruguayan Peso","rate":1013393.77},{"code":"UZS","name":"Uzbekistan Som","rate":266429752.3},{"code":"VES","name":"Venezuelan Bolívar Soberano","rate":133313.55},{"code":"VND","name":"Vietnamese Dong","rate":568675849.91},{"code":"VUV","name":"Vanuatu Vatu","rate":2880365.42},{"code":"WBTC","name":"Wrapped BTC","rate":1},{"code":"SHIB","name":"Shiba Inu","rate":2008644744.44},{"code":"WST","name":"Samoan Tala","rate":66046.44},{"code":"XAF","name":"CFA Franc BEAC","rate":15628639.67},{"code":"XAG","name":"Silver (troy ounce)","rate":1197.29},{"code":"XAU","name":"Gold (troy ounce)","rate":13.8},{"code":"XCD","name":"East Caribbean Dollar","rate":65847.25},{"code":"XPF","name":"CFP Franc","rate":2843163.21},{"code":"XOF","name":"CFA Franc BCEAO","rate":15628639.67},{"code":"YER","name":"Yemeni Rial","rate":6094871.18},{"code":"ZAR","name":"South African Rand","rate":500000.00},{"code":"ZMW","name":"Zambian Kwacha","rate":398657.44},{"code":"ZWL","name":"Zimbabwean Dollar","rate":7845485.16}]}`
        )
        
        return testData.data
    }
    return needle(
        "GET",
        `https://api.bitnob.co/api/v1/rates/exchange`, 
        {},
        {
            json: true
        }
    ).then(response => {
        if (response.statusCode == 200 && response.body.status == true) {
            return response.body.data
        } else {
            console.error("Fetch Bitnob status code: ", response.statusCode)
            console.error("Fetch Bitnob status message: ", response.statusMessage)
            console.error("Fetch Bitnob status content: ", response.body)
            return null
        }
    }).catch(reason => {
        console.error("Something went wrong fetching from bitnob: ", reason);
        return null
    })
}

const bitnobExchangeRate = async (currencyTicker, bitnobExchangeRate) => {
    if (bitnobExchangeRate) {
        var currencyPrice = bitnobExchangeRate.find(c => c.code == currencyTicker)

        if (currencyPrice) {
            return {
                currency: currencyPrice.code,
                marketExchangeRate: moneyFormat(currencyPrice.rate), 
                marketValue: currencyPrice.rate,
            }
        }
    }
    return {
        currency: "BTC",
        marketExchangeRate: "1", 
        marketValue: 1,
    };
}

const moneyFormat = (amount) => {
    return Number(amount).toLocaleString("za", {
        minimumFractionDigits: 2,
        maximumFractionDigits: 2
    });
}
