const config = require("config");
const needle = require("needle");

const kotaniAPIDomain = config.get("kotani.domain");
const kotaniAPIToken = config.get("kotani.api");

module.exports.createMobileMoneyCustomer = (phone_number, country_code) => {
    const kotaniAPICreateMobileMoneyCustomerURL = `https://${kotaniAPIDomain}/api/v3/customer/mobile-money`

    const data = {
        phone_number,
        country_code,
        // network
    }
    return needle(
        "POST",
        kotaniAPICreateMobileMoneyCustomerURL,
        data,
        {
            json: true,
            headers: {
                "Authorization": `Bearer ${kotaniAPIToken}`
            }
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to create mobile money customer: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}

module.exports.getMobileMoneyCustomer = (customer_key) => {
    const kotaniAPIGetMobileMoneyCustomerURL = `https://${kotaniAPIDomain}/api/v3/customer/mobile-money/${customer_key}`

    return needle(
        "GET",
        kotaniAPIGetMobileMoneyCustomerURL,
        {
            json: true,
            headers: {
                "Authorization": `Bearer ${kotaniAPIToken}`
            }
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to get mobile money customer: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}