const config = require("config");
const needle = require("needle");

const kotaniAPIDomain = config.get("kotani.domain");
const kotaniAPIToken = config.get("kotani.api");

module.exports.createIntegrator = (integrator) => {
    const kotaniAPICreateIntegratorURL = `https://${kotaniAPIDomain}/api/v3/integrator/`

    const data = {
        organization: integrator.organization,
        first_name: integrator.first_name,
        last_name: integrator.last_name,
        email: integrator.email,
        phone: integrator.phone,
        country_code: integrator.country_code
    }

    return needle(
        "POST",
        kotaniAPICreateIntegratorURL,
        data,
        {
            json: true,
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to get address transaction: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}


module.exports.getIntegrator = (integrator) => {
    const kotaniAPIGetIntegratorURL = `https://${kotaniAPIDomain}/api/v3/integrator/`

    return needle(
        "GET",
        kotaniAPIGetIntegratorURL,
        {
            json: true,
            headers: {
                "Authorization": `Bearer ${kotaniAPIToken}`
            }
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to get address transaction: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}