const config = require("config");
const needle = require("needle");

const kotaniAPIDomain = config.get("kotani.domain");
const kotaniAPIToken = config.get("kotani.api");
const walletId = config.get("kotani.walletId");

module.exports.withdrawToMobileMoney = (customer_key, transfer) => {
    const kotaniAPIWithdrewToMobileMoneyURL = `https://${kotaniAPIDomain}/api/v3/withdraw/mobile-money`

    const data = {
        customer_key,
        amount: transfer.kotaniPayWithdraw.amount,
        walletId,
        callbackUrl: `https://onitsha.machankura.com/kotani/${transfer.id}`,
        referenceId: transfer.id
    }

    return needle(
        "POST",
        kotaniAPIWithdrewToMobileMoneyURL,
        data,
        {
            json: true,
            headers: {
                "Authorization": `Bearer ${kotaniAPIToken}`
            }
        }
    ).then(response => {
        if (response.statusCode == 201) {
            return response.body
        } else {
            console.error("Failed to withdraw to mobile money: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}

module.exports.GetMobileMoneyWithdrawal = (reference_id) => {
    const kotaniAPIWithdrewStatusURL = `https://${kotaniAPIDomain}/api/v3/withdraw/status/${reference_id}`

    return needle(
        "GET",
        kotaniAPIWithdrewStatusURL,
        {
            json: true,
            headers: {
                "Authorization": `Bearer ${kotaniAPIToken}`
            }
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to withdraw status: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        return null
    })
}