const bolt11 = require("bolt11");
const sparkoListNodes = require("../sparko/list-nodes")

const idGenerator = require("machankura.db/src/lib/id-generator")

module.exports.machankuraOutgoingLightningTranscation = async (db, user, amount, serviceFee, receiver, bolt11Invoice, t = null) => {
    const bolt11Object = bolt11.decode(bolt11Invoice)
                            
    const lightningNode = await this.machankuraLightningNode(db, bolt11Object)
    if (lightningNode && bolt11Object.satoshis == amount) {
        const transfer = await db.Transfer.create({
            sourceId: user.id,
            amountInSatoshis: amount,
            feeInSatoshis: serviceFee,
            destinationParameter: receiver,
            status: "pending",
            bolt11Transfer: {
                id: idGenerator.generateUniqueId(),
                status: "pending",
                invoice: bolt11Invoice,
                lightninNodeId: lightningNode.id,
                amountInSatoshis: amount,
                expiresAt: new Date(bolt11Object.timeExpireDateString)
            }
        }, {
            include: [
                {
                    association: db.Transfer.Bolt11Transfer
                }
            ],
            transaction: t
        })

        return transfer
    } else {
        console.error("Couldn't store lightning node that issued: ", bolt11Invoice)
        return null
    }
}

module.exports.machankuraLightningNode = async (db, bolt11Object) => {
    const existingLightningNode = await db.LightningNode.findByPk(bolt11Object.payeeNodeKey)
    if (existingLightningNode) {
        return existingLightningNode
    } else {
        const nodeDetail = await sparkoListNodes.listNodes(bolt11Object.payeeNodeKey)
        if (nodeDetail) {
            return await db.LightningNode.create({
                id: bolt11Object.payeeNodeKey,
                alias: nodeDetail.alias
            }).catch(e => {
                console.error(`Failed to create lightning node ${bolt11Object.payeeNodeKey}: `, e)
                return null;
            })
        } else {
            return null
        }
    }
} 
