const needle = require("needle")
const { createHash, randomBytes } = require('crypto');
const bolt11 = require("bolt11");

const { bech32 } = require('bech32')

module.exports.decodeLNURL = (href) => {
    const encodedLNURL = href.split("lightning:").at(-1)
    
    const decoded = bech32.decode(encodedLNURL, 1000)
    return Buffer.from(bech32.fromWords(decoded.words)).toString("utf8")
}

const getLightningAddressJSON = (username, domain) => {
    // TODO: Support onion lightning addresses
    const lightninAddressURL = `https://${domain}/.well-known/lnurlp/${username}`
    return this.getLNURLJSON(
        lightninAddressURL
    )
}

module.exports.getLNURLJSON = (lnurl, expectedTag = "payRequest") => {
    return needle(
        "GET",
        lnurl,
        {
            json: true
        }
    ).then(response => {
        if (response.statusCode == 200 && response.body.tag == expectedTag) {
            return response.body
        } else {
            console.error("Failed to get LN service response: ", response.statusMessage)
            // throw Error(response.statusMessage)
            return null
        }
    }).catch(error => {
        return null
    })
}

module.exports.submitWithdrawalRequest = (withdrawalURL, expectedTag = "withdrawRequest") => {
    return needle(
        "GET",
        withdrawalURL,
        {
            json: true
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to submit withdrawal request: ", response.statusMessage)
            console.error("Error Body: ", response.body)
            return null
        }
    }).catch(error => {
        console.error("Failed to submit withdrawal request: ", error)
        return null
    })
}

module.exports.getLNURLJSON = (lnurl, expectedTag = "payRequest") => {
    return needle(
        "GET",
        lnurl,
        {
            json: true
        }
    ).then(response => {
        if (response.statusCode == 200 && response.body.tag == expectedTag) {
            return response.body
        } else {
            console.error("Failed to get LN service response: ", response.statusMessage)
            // throw Error(response.statusMessage)
            return null
        }
    }).catch(error => {
        return null
    })
}

