const config = require("config");
const path = require("path");
const fs = require("fs")
const needle = require("needle")
const crypto = require("crypto")

const myriadPubKeyPath = path.join(__dirname, "./resources/keys/public.key")

const domain = config.get("server.ussdDomain");
const machankuraMyriadServiceURL = `https://${domain}/myriad/ivory-coast`;

const myriadDomain = config.get("myriad-group.u-browser-domain")
const myriadSID = config.get("myriad-group.sid")
const myriadAuth = config.get("myriad-group.auth")
const myriadCustomToken = config.get("myriad-group.custom-token")
const isMyriadSandbox = config.get("myriad-group.is-sandbox")

const myriadPublicKey = fs.readFileSync(myriadPubKeyPath, "utf8");   // Importing public key

module.exports.myriadEncrypt = function (plainText) {
    // open public key file and read key
    console.log(`Encrypting: ${plainText}.`)
    const cipherText = crypto.publicEncrypt(                   // Ecrypting plain text
        {
            key: myriadPublicKey,
            padding: crypto.constants.RSA_PKCS1_PADDING,       // using PKCS1 padding
        }, 
        Buffer.from(plainText)
    );                                   // with the auth token

    return cipherText.toString("base64");                      // return the encrypted token
}

module.exports.validatePhoneNumber = (phoneNumber, alias) => {
    let myriadValidateMSISDNUrl = `https://${myriadDomain}/api/v1.0/validate?tested_msisdn=${phoneNumber}&alias=${alias}`

    console.log("Calling: ", myriadValidateMSISDNUrl);
    return needle(
        "GET",
        myriadValidateMSISDNUrl,
        {
            json: true,
            headers: {
                "sid": myriadSID,
                "auth": myriadAuth
            }
        }
    ).then(response => {
        if (response.statusCode == 200) {
            console.log("Validate MSISDN: ", response.body);
            return response.body
        } else {
            console.error("Failed to validate MSISIDN: ", response.statusMessage)
            console.log("Status code: ", response.statusCode)
            console.log("Created Body: ", response.body)
            console.log("Headers: ", response.headers)

            return null
        }
    }).catch(error => {
        console.error("Error calling validate MSISDN: ", error)
        return null
    })
}

module.exports.getMyriadSID = () => { 
    return myriadSID
}
module.exports.getMyriadAuth = () => {
    return myriadAuth
}
module.exports.getMyriadCustomToken = () => {
    return myriadCustomToken
}

module.exports.getCountryCallingCodeFromMyriadCountryCode = (myriadUserCountry) => {
    // TODO: Get all country codes and map them to the country calling code...
    if (myriadUserCountry == "384") {
        return "225"
    } else if (myriadUserCountry == "777") { // For testing...
        return "225"
    }  
    return "21"
}

module.exports.myriadNormalizePhoneNumber = (phoneNumber, myriadUserCountry) => {
    if (phoneNumber.startsWith("0")) {
        return phoneNumber.replace(
            "0",
            this.getCountryCallingCodeFromMyriadCountryCode(myriadUserCountry)
        ).replaceAll(" ", "").replaceAll("-", "")
    }
    
    return phoneNumber.replace("+","").replaceAll(" ", "").replaceAll("-", "")
}

module.exports.myriadInitiateFormResponse = (text, endSession = false) => {
    const data = {
        message: text,
        page: {
            history: false,
            menu: false,
            volatile: true,
            navigation_keywords: false
            // TODO: Support language...
            // TODO: Callback for activity things...
        }
    };

    if (endSession) {
        data.page.session_end = true
    } else {
        data.form = {
            "url": machankuraMyriadServiceURL,
            "type": "text",
            "method": "get"
        }
    }
    
    return data
}