const machankuraSMS = require('machankura.sms')
const queryCountry = require("../../queries/country")
const phoneUtil = require('google-libphonenumber').PhoneNumberUtil.getInstance();

module.exports.notifyBitrefillVoucherDelivered = async (db, bitrefillInvoice, userId) => {
    const user = await db.User.findByPk(userId, {
        include: [
            {
                association: db.User.UserPhones,
                required: true,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        require: true
                    }
                ]
            }
        ]
    })
    if (user) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendBitrefillVoucherSuccess(
                country,
                user.userPhones[0].phone.number,
                bitrefillInvoice
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`, bitrefillInvoice.toJSON())
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyFailedBitrefillVoucherDelivery = async (db, bitrefillInvoice, userId) => {
    const user = await db.User.findByPk(userId, {
        include: [
            {
                association: db.User.UserPhones,
                required: true,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        require: true
                    }
                ]
            }
        ]
    })
    if (user) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendFailedToDeliverBitrefillVoucher(
                country,
                user.userPhones[0].phone.number,
                bitrefillInvoice
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}