const machankuraSMS = require('machankura.sms')
const queryCountry = require("../queries/country")
const phoneUtil = require('google-libphonenumber').PhoneNumberUtil.getInstance();

const MINIMUM_SMS_AMOUNT_IN_SATS = 100

module.exports.normalizeReceiverNumber = (user, receiver) => {
    if (receiver.startsWith("0")) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()
        return `+${receiver.replace("0", countryCallingCode)}`
    } else if (!receiver.startsWith("+")) {
        return `+${receiver}`
    } else {
        return receiver
    }
}

module.exports.normalizeReceiverNumberByCountry = (country, receiver) => {
    if (receiver.startsWith("0")) {
        return `+${receiver.replace("0", country.countryCallingCode)}`
    } else if (!receiver.startsWith("+")) {
        return `+${receiver}`
    } else {
        return receiver
    }
}

module.exports.notifyLightingGiftedUserBySMS = async (db, receiver, amount) => {
    try {
        // TODO: Store SMS in DB
        // TODO: Only notify when amount > 10000
        if (amount > MINIMUM_SMS_AMOUNT_IN_SATS) {
            const countryCallingCode = phoneUtil.parse(`+${receiver}`).getCountryCode()

            const country = await queryCountry.getServicableCountryByCallingCode(
                db,
                countryCallingCode
            )

            if (country) {
                const result = await machankuraSMS.sendGiftedBitcoinSMS(
                    country,
                    receiver,
                    amount,
                    "sats",
                    `lightning`
                )
                console.log("Lightning Gift SMS Result: ", result?.SMSMessageData)
            } else {
                console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receiver}`)
            }
        }
    } catch (e) {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error(`Couldn't notify user ${receiver} from lighting`, e)
        }
    }
}

module.exports.notifyAffiliatePayoutBySMS = async (db, receiver, amount) => {
    try {
        // TODO: Store SMS in DB
        // TODO: Only notify when amount > 10000
        if (amount >= 1 /* TODO: Replace with MINIMUM_SMS_AMOUNT_IN_SATS */) {
            const countryCallingCode = phoneUtil.parse(`+${receiver}`).getCountryCode()

            const country = await queryCountry.getServicableCountryByCallingCode(
                db,
                countryCallingCode
            )

            if (country) {
                const result = await machankuraSMS.sendAffiliatePayoutSMS(
                    country,
                    receiver,
                    amount,
                    "sats"
                )
                console.log("Payout Gift SMS Result: ", result)
            } else {
                console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receiver}`)
            }
        }
    } catch (e) {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error(`Couldn't notify user ${receiver} about payout`, e)
        }
    }
}


module.exports.notifyGiftedUserBySMS = async (db, senderId, receiver, amount) => {
    try {
        // TODO: Store SMS in DB
        // TODO: Only notify when amount > 10000
        if (amount > MINIMUM_SMS_AMOUNT_IN_SATS) {
            const user = await db.User.findByPk(senderId, {
                include: [
                    {
                        association: db.User.UserPhones,
                        include: [
                            {
                                association: db.UserPhone.Phone
                            }
                        ]
                    },
                    {
                        association: db.User.UserName,
                        include: [
                            {
                                association: db.UserName.Name
                            }
                        ]
                    }
                ]
            })
            if (user && user.userPhones.length > 0) {
                const normalizedReceiver = this.normalizeReceiverNumber(user, receiver)
    
                const countryCallingCode = phoneUtil.parse(`${normalizedReceiver}`).getCountryCode()
    
                const country = await queryCountry.getServicableCountryByCallingCode(
                    db,
                    countryCallingCode
                )
    
                if (country) {
                    const result = await machankuraSMS.sendGiftedBitcoinSMS(
                        country,
                        normalizedReceiver,
                        amount,
                        "sats",
                        `${user.userName?.name?.text ?? ""} (+${user.userPhones[0].phone.number})`
                    )
                    // console.log("SMS Result: ", result)
                } else {
                    console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${normalizedReceiver}`)
                }
                
            } else {
                console.error(`Failed to find sender ${senderId} with the for the receiver `, receiver)
            }
        }
        
        
    } catch (e) {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error(`Couldn't notify user ${receiver} from ` + senderId, e)
        }
    }
}

module.exports.notifyUserBySMS = async (db, userId, amount, source) => {
    try {
        // TODO: Store SMS in DB
        // TODO: Only notify when amount > 10000
        if (amount > MINIMUM_SMS_AMOUNT_IN_SATS || source == "azte.co") {
            const user = await db.User.findByPk(userId, {
                include: [
                    {
                        association: db.User.UserPhones,
                        include: [
                            {
                                association: db.UserPhone.Phone
                            }
                        ]
                    }
                ]
            })
            if (user && user.userPhones.length > 0) {
                const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()
    
                const country = await queryCountry.getServicableCountryByCallingCode(
                    db,
                    countryCallingCode
                )
    
                if (country) {
                    machankuraSMS.sendReceivedTransactionSMS(
                        country,
                        user.userPhones[0].phone.number,
                        amount,
                        "sats",
                        source
                    )
                } else {
                    console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
                }
                
            } else {
                console.error("Failed to find user with the id: ", userId)
            }
        }
        
        
    } catch (e) {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error("Couldn't notify user " + userId, e)
        }
    }
    
}

module.exports.remindUserOfPinResetBySMS = async (db, userPinResetRequest) => {
    try {
        // TODO: Store SMS in DB
        const user = userPinResetRequest.user
        if (user && user.userPhones.length > 0) {
            const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

            const country = await queryCountry.getServicableCountryByCallingCode(
                db,
                countryCallingCode
            )

            if (country) {
                if (userPinResetRequest.activatesAt > new Date()) {
                    return machankuraSMS.sendPendingPinResetReminder(
                        country,
                        user.userPhones[0].phone.number,
                        userPinResetRequest
                    )
                } else {
                    return machankuraSMS.sendActivePinResetReminder(
                        country,
                        user.userPhones[0].phone.number,
                        userPinResetRequest
                    )
                }
                
            } else {
                console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
            }
            
        } else {
            console.error("Failed to find user with the id: ", userId)
        }
        
    } catch (e) {
        console.error("Couldn't remind user of pending pin reset " + userId, e)
    }
    
    return null
}

module.exports.notifyAztecoFailed1ForYouConversion = async (db, redeemableForAztecoVoucher) => {
    const user = await db.User.findByPk(redeemableForAztecoVoucher.userId, {
        include: [
            {
                association: db.User.UserPhones,
                include: [
                    {
                        association: db.UserPhone.Phone
                    }
                ]
            }
        ]
    })
    if (user && user.userPhones.length > 0) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendAzteco1ForYouConversionFailureSMS(
                country,
                user.userPhones[0].phone.number,
                redeemableForAztecoVoucher
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyAztecoFailedLightningWithdrawalSubmission = async (db, aztecoVoucher) => {
    const user = await db.User.findByPk(aztecoVoucher.userId, {
        include: [
            {
                association: db.User.UserPhones,
                include: [
                    {
                        association: db.UserPhone.Phone
                    }
                ]
            }
        ]
    })
    if (user && user.userPhones.length > 0) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendAztecoLightningWithdrawalFailureSMS(
                country,
                user.userPhones[0].phone.number,
                aztecoVoucher
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyAztecoOnchainWithdrawalUnsupported = async (db, aztecoVoucher) => {
    const user = await db.User.findByPk(aztecoVoucher.userId, {
        include: [
            {
                association: db.User.UserPhones,
                include: [
                    {
                        association: db.UserPhone.Phone
                    }
                ]
            }
        ]
    })
    if (user && user.userPhones.length > 0) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendAztecoOnchainWithdrawalUnsupportedSMS(
                country,
                user.userPhones[0].phone.number,
                aztecoVoucher
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyUserOfRegistrationBySMS = async (db, receiver) => {
    const countryCallingCode = phoneUtil.parse(`+${receiver}`).getCountryCode()

            const country = await queryCountry.getServicableCountryByCallingCode(
                db,
                countryCallingCode
            )

            if (country) {
                const result = await machankuraSMS.sendWelcomeToMachankuraSMS(
                    country,
                    receiver
                )
            } else {
                if (process.env.JEST_WORKER_ID == undefined) {
                    console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receiver}`)
                }
            }
}

module.exports.notifyWattsDelivered = async (db, cryptoConvertWattsVoucher, userId) => {
    const user = await db.User.findByPk(userId, {
        include: [
            {
                association: db.User.UserPhones,
                required: true,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        require: true
                    }
                ]
            }
        ]
    })
    if (user && user.userPhones.length > 0) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendWattsVoucherSuccess(
                country,
                user.userPhones[0].phone.number,
                cryptoConvertWattsVoucher
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyFailedWattsDelivery = async (db, cryptoConvertWattsInvoice, userId) => {
    const user = await db.User.findByPk(userId, {
        include: [
            {
                association: db.User.UserPhones,
                required: true,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        require: true
                    }
                ]
            }
        ]
    })
    if (user && user.userPhones.length > 0) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendFailedToDeliverWattsVoucher(
                country,
                user.userPhones[0].phone.number,
                cryptoConvertWattsInvoice
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyUserOfTemporaryOTP = async (db, receiver, otp) => {
    const countryCallingCode = phoneUtil.parse(`+${receiver}`).getCountryCode()

    const country = await queryCountry.getServicableCountryByCallingCode(
        db,
        countryCallingCode
    )

    if (country) {
        return await machankuraSMS.sendMachankuraOTP(
            country,
            receiver,
            otp
        )
    } else {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receiver}`)
        }
    }
}

module.exports.notifyUserOfSubwayChallenge = async (db, receiver, challengeContent) => {
    const countryCallingCode = phoneUtil.parse(`+${receiver}`).getCountryCode()

    const country = await queryCountry.getServicableCountryByCallingCode(
        db,
        countryCallingCode
    )

    if (country) {
        return await machankuraSMS.sendSubwayChallenge(
            country,
            receiver,
            challengeContent
        )
    } else {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receiver}`)
        }
    }
}

module.exports.notifyTransferFailedPaymentProcessing = async (db, transfer) => {
    const user = await db.User.findByPk(transfer.sourceId, {
        include: [
            {
                association: db.User.UserPhones,
                include: [
                    {
                        association: db.UserPhone.Phone
                    }
                ]
            }
        ]
    })
    if (user && user.userPhones.length > 0) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendTransferPaymentProcessorFailureSMS(
                country,
                user.userPhones[0].phone.number,
                transfer
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`)
        }
        
    } else {
        console.error("Failed to find user who is the source for the transer: ", transfer.id)
    }
}


module.exports.broadcastSMS = async (country, receivers, message) => {

    if (country) {
        return await machankuraSMS.broadcastSMS(
            country,
            receivers,
            message
        )
    } else {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receivers}`)
        }
    }
}

module.exports.notifyVendorsInviteSMS = async (db, receiver) => {
    try {
        const countryCallingCode = phoneUtil.parse(`+${receiver}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )
    
        if (country) {
            return await machankuraSMS.sendVendorsInviteSMS(
                country,
                receiver,
                "Digits for Vendors",
            )
        } else {
            if (process.env.JEST_WORKER_ID == undefined) {
                console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receiver}`)
            }
        }
    } catch (e) {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error("Couldn't notify invitee " + receiver, e)
        }
    }
}

module.exports.notifyVendorsVoucherSMS = async (db, receiver, voucher) => {
    try {
        const countryCallingCode = phoneUtil.parse(`+${receiver}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )
    
        if (country) {
            return await machankuraSMS.sendVendorsVoucherSMS(
                country,
                receiver,
                "Digits for Vendors",
                voucher
            )
        } else {
            if (process.env.JEST_WORKER_ID == undefined) {
                console.error(`Failed to find country with the country calling code ${countryCallingCode} for receiver ${receiver}`)
            }
        }
    } catch (e) {
        if (process.env.JEST_WORKER_ID == undefined) {
            console.error("Couldn't notify invitee " + receiver, e)
        }
    }
}

module.exports.bitrefill = require("./bitrefill")
module.exports.onitsha = require("./onitsha")
module.exports.clan = require("./clan")