const machankuraSMS = require('machankura.sms')
const queryCountry = require("../../queries/country")
const phoneUtil = require('google-libphonenumber').PhoneNumberUtil.getInstance();

module.exports.notifyPaymentReceived = async (db, userId, payment) => {
    const user = await db.User.findByPk(userId, {
        include: [
            {
                association: db.User.UserPhones,
                required: true,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        require: true
                    }
                ]
            }
        ]
    })
    if (user) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendOnchainPaymentReceived(
                country,
                user.userPhones[0].phone.number,
                payment
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`, payment.toJSON())
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyPaymentConfirmed = async (db, userId, payment) => {
    const user = await db.User.findByPk(userId, {
        include: [
            {
                association: db.User.UserPhones,
                required: true,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        require: true
                    }
                ]
            }
        ]
    })
    if (user) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendOnchainPaymentConfirmed(
                country,
                user.userPhones[0].phone.number,
                payment
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`, payment.toJSON())
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}

module.exports.notifyBitcoinAddress = async (db, userId, onchainAddress) => {
    const user = await db.User.findByPk(userId, {
        include: [
            {
                association: db.User.UserPhones,
                required: true,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        require: true
                    }
                ]
            }
        ]
    })
    if (user) {
        const countryCallingCode = phoneUtil.parse(`+${user.userPhones[0].phone.number}`).getCountryCode()

        const country = await queryCountry.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )

        if (country) {
            machankuraSMS.sendOnchainBitcoinAddress(
                country,
                user.userPhones[0].phone.number,
                onchainAddress
            )
        } else {
            console.error(`Failed to find country with the country calling code ${countryCallingCode} for user ${userId}`, onchainAddress)
        }
        
    } else {
        console.error("Failed to find user with the id: ", userId)
    }
}