const phoneNumberUtil = require("google-libphonenumber").PhoneNumberUtil.getInstance()

module.exports.getServicableCountryByCallingCode = async function (db, countryCallingCode) {
    return db.Country.findOne({
        where: {
            countryCallingCode: countryCallingCode
        },
        include: [
            {
                association: db.Country.ServiceCountry,
                required: true,
                include: [
                    {
                        association: db.ServiceCountry.AfricasTalking,
                    },
                    {
                        association: db.ServiceCountry.UssdServices
                    }
                ]
            }
        ]
    }).catch(error => {
        console.error("Failed to find servicable country: ", error)
        return null
    })
}

module.exports.getServicableCountryByPhoneNumber = async function (db, phoneNumber) {
    try {
        const countryCallingCode = phoneNumberUtil.parse(`+${phoneNumber}`).getCountryCode()
        return await this.getServicableCountryByCallingCode(
            db,
            countryCallingCode
        )
    } catch(e){
        console.log("Error getServicableCountryByPhoneNumber:", e)
    }
    return null
}

module.exports.isPhoneNumbersCountrySupported = async (db, phoneNumber) => {
    const country = await this.getServicableCountryByPhoneNumber(
        db,
        phoneNumber
    )

    return ["GENERALLY_AVAILABLE"].includes(country?.serviceCountry?.status)// TODO: Make this internally avaialble 
}