const config = require("config")
const reggie = require("../../reggie")

const domain = config.get("server.domain")

const isMachankuraLightningAddress = function(receiver) {
    return receiver.match(reggie.lightningAddressRegex) && receiver.split('@')[1]?.toLowerCase() == "8333.mobi"
}

const getUserIdentifier = function (receiver) {
    if (receiver.match(reggie.phoneNumberRegex) || receiver.toLowerCase().match(reggie.userNameRegex)) {
        return receiver.toLowerCase()
    } else if (isMachankuraLightningAddress(receiver)) {
        return getUserIdentifier(receiver.split('@')[0])
    } else {
        return null
    }
}


module.exports.getUser = async function (db, receiver, associations = null) {
    const userId = getUserIdentifier(receiver)

    const userIncludes = associations ? associations : []

    userIncludes.push({
        association: db.User.Wallet,
        include: [
            {
                association: db.Wallet.WalletName,
                include: [
                    {
                        association: db.WalletName.Name
                    }
                ]
            }
        ]
    })
    if (userId?.match(reggie.phoneNumberRegex)) {
        userIncludes.push(
            {
                association: db.User.UserPhones,
                include: [
                    {
                        association: db.UserPhone.Phone,
                        where: {
                            number: userId.trim()
                        }
                    }
                ]
            }
        )

        return db.User.findOne({
            include: userIncludes
        })
    } else if (userId?.match(reggie.userNameRegex)) {
        userIncludes.push(
            {
                association: db.User.UserName,
                required: true,
                include: [
                    {
                        association: db.UserName.Name,
                        required: true,
                        where: {
                            text: userId.trim().toLowerCase()
                        }
                    }
                ]
            }
        )
        return db.User.findOne({
            include: userIncludes
        })
    }

    return null
}