const phoneNumberRegexString = "[\\+]?[(]?[0-9]{3}[)]?[-\\s\\.]?[0-9]{3}[-\\s\\.]?[0-9]{4,7}"

const userNameRegexString = "[a-z][a-z0-9_-]{2,14}"

const lightningAddressRegexString = "[^@ \\t\\r\\n]+@[^@ \\t\\r\\n]+\\.[^@ \\t\\r\\n\\*]+"
const webDomainRegexString = "[-a-zA-Z0-9@:%._\\+~#=]{1,256}\\.[a-zA-Z0-9()]{1,6}"
const bitcoinAddressRegexString = "(bc1|[13])[a-zA-HJ-NP-Z0-9]{25,63}"
const bolt11RegexString = "lnbc.+"

const machankuraReceiverRegexString = `(${phoneNumberRegexString}|${userNameRegexString}|${lightningAddressRegexString}|${bitcoinAddressRegexString})`

const pinRegexString = "\\d{4,8}"
const selectionRegexString = "[1-9]"

const navigateToMenuRegexString = "99($|\\*)"

const aztecoCodeRegexString = `[0-9]{16}`

module.exports.machankuraReceiverRegex = new RegExp(machankuraReceiverRegexString)
module.exports.aztecoHyperlinkRegex = /href="(lightning:.*|https:\/\/azte\.co\/redeem\?code=[0-9]{16})/
module.exports.aztecoOnchainCodeRegex = /https:\/\/azte\.co\/redeem\?code=([0-9]{16})/
module.exports.aztecoCodeRegex = new RegExp(aztecoCodeRegexString)
module.exports.whatIsMachankuraRegex = new RegExp(`^2$`)
module.exports.redeemAztecoCodeEnteredRegex = new RegExp(`^${aztecoCodeRegexString}$`)
module.exports.whatIsBitcoinRegex = new RegExp(`^2\\*1$`)
module.exports.changeLanguageRegex = new RegExp(`^3$`)
module.exports.phoneNumberRegex = new RegExp(`^${phoneNumberRegexString}$`)
module.exports.userNameRegex = new RegExp(`^${userNameRegexString}$`)
module.exports.lightningAddressRegex = new RegExp(`^${lightningAddressRegexString}$`)
module.exports.bitcoinAddressRegex = new RegExp(`^${bitcoinAddressRegexString}$`)
module.exports.bolt11Regex = new RegExp(`^${bolt11RegexString}$`)
module.exports.sendBitcoinCurrencySelectedRegex = new RegExp("^1\\*[1-2]$")
module.exports.sendBitcoinEnteredAmountRegex = new RegExp("^1\\*[1-2]\\*\\d{0,10}$")
module.exports.sendBitcoinEnteredMethodRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*[1-3]$`)
module.exports.sendBitcoinEnteredQuickMethodsRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*(11|12)$`)
module.exports.sendBitcoinEnteredQuickMethodSelectedRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*(11|12)\\*[1-9]$`)
module.exports.sendBitcoinEnteredQuickMethodPinRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*(11|12)\\*[1-9]\\*${pinRegexString}$`)
module.exports.sendBitcoinEnteredReceiverRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*(1|3)\\*${machankuraReceiverRegexString}$`)
module.exports.sendBitcoinEnteredFullLightningAddressRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*2\\*${lightningAddressRegexString}$`)
module.exports.sendBitcoinEnteredHalfLightningAddressRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*2\\*${machankuraReceiverRegexString}$`)
module.exports.sendBitcoinEnteredSecondHalfLightningAddressRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*2\\*${machankuraReceiverRegexString}\\*${webDomainRegexString}$`)
module.exports.sendBitcoinEnteredPinRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*[1-3]\\*${machankuraReceiverRegexString}\\*${pinRegexString}$`)
module.exports.sendBitcoinSplitLightningAddressEnteredPinRegex = new RegExp(`^1\\*[1-2]\\*\\d{0,10}\\*2\\*${machankuraReceiverRegexString}\\*${webDomainRegexString}\\*${pinRegexString}$`)
module.exports.receiveBitcoinRegex = new RegExp(`^2$`)
module.exports.receiveBitcoinViaAztecoCodeEnteredRegex = new RegExp(`^2\\*1\\*${aztecoCodeRegexString}$`)
module.exports.receiveBitcoinViaAztecoCodeOptionEnteredRegex = new RegExp(`^2\\*1\\*${aztecoCodeRegexString}\\*[1-2]$`)
module.exports.receiveBitcoinQRCodeRegex = new RegExp(`^2\\*3$`)
module.exports.accountDetialsEnteredPinOpenEndedRegex = new RegExp(`^3\\*${pinRegexString}`)
module.exports.accountDetialsExchangeRateRegex = new RegExp(`^3\\*${pinRegexString}\\*2$`)
module.exports.accountDetialsListTransactionsRegex = new RegExp(`^3\\*${pinRegexString}\\*1(\\*0)*$`)
module.exports.accountDetialsViewTransactionRegex = new RegExp(`^3\\*${pinRegexString}\\*1(\\*0)*\\*[1-9]$`)
module.exports.updateUsernameEnteredPinOpenEndedRegex = new RegExp(`^0\\*1\\*${pinRegexString}`)
module.exports.registerAccountEnteredPinRegex = new RegExp(`^1\\*${pinRegexString}$`)
module.exports.changePinCurrentPinEnteredRegex = new RegExp(`^0\\*2\\*1\\*${pinRegexString}$`)
module.exports.changePinNewPinEnteredRegex = new RegExp(`^0\\*2\\*1\\*${pinRegexString}\\*${pinRegexString}$`)
module.exports.resetPinNewPinEnteredRegex = new RegExp(`^0\\*2\\*${pinRegexString}$`)
module.exports.buyScreenRegex = new RegExp(`^4$`)
module.exports.buyCategoryEnteredRegex = new RegExp(`^4\\*${selectionRegexString}`)
module.exports.buyAmountEnteredRegex = new RegExp(`^4\\*${selectionRegexString}\\*[1-2](\\*${phoneNumberRegexString}){0,1}\\*\\d*`)
module.exports.buyAirtimeCompleteRegex = new RegExp(`^4\\*${selectionRegexString}\\*[1-2](\\*${phoneNumberRegexString}){0,1}(\\*[1-9]){0,1}\\*\\d*\\*${pinRegexString}`)
module.exports.exchnageRateScreenRegex = new RegExp(`^5$`)
module.exports.orangePillerScreenRegex = new RegExp(`^0\\*21$`)

const navigateableRegulaExpressions = (user) => {
    if (user) {
        return [
            this.registerAccountEnteredPinRegex,
            this.sendBitcoinEnteredPinRegex,
            this.sendBitcoinSplitLightningAddressEnteredPinRegex,
            this.sendBitcoinEnteredQuickMethodPinRegex,
            this.receiveBitcoinRegex,
            this.receiveBitcoinQRCodeRegex,
            this.accountDetialsEnteredPinOpenEndedRegex,
            this.accountDetialsExchangeRateRegex,
            this.accountDetialsListTransactionsRegex,
            this.accountDetialsViewTransactionRegex,
            this.buyScreenRegex,
            this.buyAmountEnteredRegex,
            this.buyAirtimeCompleteRegex,
            this.exchnageRateScreenRegex,
            this.orangePillerScreenRegex
        ]
    } else {
        return [
            this.whatIsMachankuraRegex,
            this.whatIsBitcoinRegex,
            this.changeLanguageRegex
        ]
    }
    
}

const replaceRegularExpression = (regularExpression) => {
    if (regularExpression.source.endsWith("$")) {
        return /\$$/ 
    } else {
        return /$/
    }
}

module.exports.transformToNavigateableRegularExpressionString = (regularExpression) => {
    return regularExpression.source.replace(
        replaceRegularExpression(regularExpression),
        `\\*${navigateToMenuRegexString}`
    )
}

module.exports.canNavigateToMenu = (user, text) => {
    // "".endsWith("*99")
    return navigateableRegulaExpressions(user).some(regularExpress => {
        return removeWasteCharactersFromPhoneNumberString(text).match(
            new RegExp(
                this.transformToNavigateableRegularExpressionString(regularExpress)
            )
        ) 
    })
}

const removeWasteCharactersFromPhoneNumberString = (str) => {
    return str.replace("+", "").replaceAll(" ", "").toLowerCase()
}

module.exports.metadataRegex = /"metadata":"\[\[.*\]\]"/