const { expect } = require('@jest/globals');
const azteco = require("../../azteco")
const nock = require('nock');

// beforeEach(async () => {
    
// });

// afterEach(async () => {
    
// });

beforeAll(async () => {
    // Return nothing for a unknown invoice
    const redemptionSuccess = nock('https://azte.co')
    .get('/1foryou_to_azteco.php')
    .query({
        CODE: "1234123412341234"
    })
    .reply(200, "4321432143214321", { 'Content-Type': 'text/plain' });

    const redemptionRetrySuccess = nock('https://azte.co')
    .get('/1foryou_to_azteco.php')
    .query({
        CODE: "1114111411141114"
    })
    .times(2)
    .reply(500, "Internal Server Error", { 'Content-Type': 'text/plain' })
    .get('/1foryou_to_azteco.php')
    .query({
        CODE: "1114111411141114"
    })
    .reply(200, "4111411141114111", { 'Content-Type': 'text/plain' });

    const redemptionFailure = nock('https://azte.co')
    .get('/1foryou_to_azteco.php')
    .query({
        CODE: "1111111111111111"
    })
    .reply(500, "Internal Server Error", { 'Content-Type': 'text/plain' }).persist();
})

test('Success Azteco redeem 1 For you', async () => {
    const result = await azteco.redeem1ForYouToAztecoVoucher({
        code: "1234123412341234"
    })

    expect(result).not.toBeNull()
    expect(result).not.toBeUndefined()
    expect(result).toBe("4321432143214321") 
});

test('Success after Azteco redeem 1 For you retries', async () => {
    const result = await azteco.redeem1ForYouToAztecoVoucher({
        code: "1114111411141114"
    })

    expect(result).not.toBeNull()
    expect(result).not.toBeUndefined()
    expect(result).toBe("4111411141114111") 
});

test('Fail to redeem 1 For you ', async () => {
    const result = await azteco.redeem1ForYouToAztecoVoucher({
        code: "1111111111111111"
    })

    expect(result).not.toBeUndefined()
    expect(result).toBeNull()
});